#!/usr/bin/perl -w
# -*- cperl -*-
#
# convert a list of gnaural XML files to parameter tables for slm
use strict;
use XML::Simple qw(:strict);

my @table;

for my $fn (@ARGV) {
  my $g = XMLin($fn, ForceArray => 1, KeyAttr => []);

  if($g->{loops}->[0] != 1) {
    die "$fn: can't handle 'loops' != 1\n";
  }
  my $firstbeat;
  my @sequence = ($fn);
  for my $v (@{$g->{voice}}) {
    next unless $v->{type}->[0] == 0; # binaural beat, 1 is noise
    for my $e (@{$v->{entries}->[0]->{entry}}) {
      my($d,$b) = ($e->{duration}, $e->{beatfreq});
      $firstbeat = $b unless defined $firstbeat;
      push(@sequence, $b, $d);
    }
  }
  push(@sequence, $firstbeat);
  push(@table, \@sequence);
}

print "// auto-generated by $0\n// " . scalar(localtime(time())) . "\n";

my $bwtab = "const struct bwentry PROGMEM * bwtab[] PROGMEM = {\n";

my $i = 1;
foreach my $t (@table) {
  my $fn = shift @$t;
  print "// from $fn\n";
  print "struct bwentry const bwtab_$i\[] PROGMEM = {\n";
  $bwtab .= "  bwtab_$i, // $fn\n";

  my $s = shift @$t;
  while(@$t) {
    my $d = shift @$t;
    my $e = shift @$t;
    printf("  { DDS_TUNE(BASE_BEAT + $s), BWSTEP(%.3f, %.3f, $d), DURATION($d) },\n",
	   $s, $e);
    $s = $e;
  }
  print "  { 0,0,0 }\n";
  print "};\n\n";
  $i++;
}

print $bwtab, "};\n";
